#!/usr/bin/env runhaskell

import           Data.SCAD
import           Data.Shared
import           Data.SRDF.Xml                     (writeSRDFXML)
import           Data.URDF.Xml                     (writeXML)
import           Data.Walker
import           Linear.Quaternion
import           Linear.V3
import           Numeric.Units.Dimensional.Prelude ((*~), degree, radian, second)
import qualified Numeric.Units.Dimensional.Prelude as D

data LegPos = RL | RR | FR | FL deriving Eq;



kokeTipPos = V3 (90 *~ mm) (0 *~ mm) (0 *~ mm)
femurOffset :: Pose
femurOffset = Pose (V3 (61.3 *~ mm) (0 *~ mm) (0 *~ mm)) $ toQuat zAxis (0 *~ degree)
tibiaOffset :: Pose
tibiaOffset = Pose (V3 (86 *~ mm) (0 *~ mm) (0 *~ mm)) $ toQuat yAxis (90 *~ degree)

legPose :: LegPos -> Pose
legPose pos = Pose (V3 x y (0 *~ mm)) $ toQuat zAxis ( angle *~ degree )
    where
        x = case pos of
            FR -> 36 *~ mm
            RR -> 36 *~ mm
            FL -> (-36) *~ mm
            RL -> (-36) *~ mm
        y = case pos of 
            FL -> 45 *~ mm
            FR -> 45 *~ mm 
            RR -> (-45) *~ mm
            RL -> (-45) *~ mm

        angle = standingZAngles pos 

standingZAngles :: LegPos -> Double
standingZAngles pos = case pos of
            RL -> -90 - 45
            RR -> -90 + 45
            FL -> 90 + 45 
            FR -> 90 - 45 

foldedZAngles :: LegPos -> Double
foldedZAngles pos = case pos of 
            RL -> -90
            RR -> -90
            FL -> 90
            FR -> 90

tibia = Part {
        model = scadModel "tibia" [ ("tipOffset",kokeTipPos) ] ["tibia.scad"],
        collision_models = [
            scadModel "tibia_collision" [("tipOffset", kokeTipPos)] ["tibia.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }


femur = Part {
        model = scadModel "femur" [ ( "tibiaOffset", pos $ tibiaOffset)]  ["femur.scad"],
        collision_models = [
            scadModel "femur_collision_a" [("tibiaOffset", pos $ tibiaOffset)] ["femur.scad"],
            scadModel "femur_collision_b" [("tibiaOffset", pos $ tibiaOffset)] ["femur.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }

coxa = Part {
        model = scadModel "coxa" [ ( "femurOffset", pos $ femurOffset)] ["coxa.scad"],
        collision_models = [
            scadModel "coxa_collision" [ ( "femurOffset", pos $ femurOffset)] ["coxa.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }

kokeBody legs = Body $ Part {
        model = scadModel "body" [("positions", map leg_pose legs)] ["body.scad"],
        collision_models = [
            scadModel "body_collision_a" [("positions", map leg_pose legs)] ["body.scad"]
        ],
        part_weight = Nothing,
        center_of_mass = Nothing
    }
    where
        leg_pose leg = pos $ offset $ head $ chain leg


leg :: LegPos -> Leg
leg pos = Leg {
        leg_name = case pos of
            RL -> "rl"
            RR -> "rr"
            FL -> "fl"
            FR -> "fr"
        ,
        chain = [
            Link { link_name = "coxa",
             offset=  legPose pos,
             part = coxa,
             axis =  zAxis,
            joint_config = JointConfig{
                min_angle = (-3*pi/8) *~ radian,
                default_angle =  0 *~ radian,
                max_angle =  (3*pi/8) *~ radian,
                vel_lim =  50 *~ (radian D./ second),
                effor_lim =  100,
                g_steps = 8
                }},
            Link { link_name = "femur", 
                    offset = femurOffset,
                    part = femur,
                    axis = yAxis,
                joint_config = JointConfig {
                    min_angle=(-3*pi/8) *~ radian,
                    default_angle=(-pi/8) *~ radian,
                    max_angle=(3*pi/8) *~ radian,
                    vel_lim= 50 *~ (radian D./ second),
                    effor_lim = 100,
                    g_steps= 8
                }},
            Link {
                link_name ="tibia",
                offset = tibiaOffset,
                part = tibia,
                axis = yAxis, 
                joint_config = JointConfig {
                    min_angle = (-3*pi/8) *~ radian,
                    default_angle = (pi/8) *~ radian,
                    max_angle = (3*pi/8) *~ radian,
                    vel_lim = 50 *~ (radian  D./ second),
                    effor_lim = 100,
                    g_steps = 8 
                }
            }
        ],
        tip = Tip {
            point = kokeTipPos
        },
        pose = noOffset --  is not considered during offset in schpin_ompl -> remove
    }

koke :: Walker
koke = Walker walker_name legs (kokeBody legs)
    where
        walker_name = "Tote"
        legs = [leg RR, leg RL, leg FR, leg FL]

main :: IO ()
main = do
   mapM_ (uncurry renderSCAD) $ getSCAD koke
   writeXML (getURDF koke) "src/urdf/koke.urdf"
   writeSRDFXML (getSRDF koke) "src/srdf/koke.srdf"
   renderWalkerStl koke
